import React, { PropTypes } from 'react'
import { findDOMNode } from 'react-dom'
import { Flex, Item } from 'react-flex'
import Component from 'react-class'

import DragHelper from 'drag-helper'
import assign from 'object-assign'

import join from './join'

const CLASS_NAME = 'react-split-container__splitter'

const emptyFn = () => {}

const ARROWS = {
  right: <svg fill="#000000" height="22" width="22" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">
    <path d="M8.59 16.34l4.58-4.59-4.58-4.59L10 5.75l6 6-6 6z"/>
    <path d="M0-.25h24v24H0z" fill="none"/>
  </svg>,
  left: <svg fill="#000000" height="22" width="22" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">
    <path d="M15.41 16.09l-4.58-4.59 4.58-4.59L14 5.5l-6 6 6 6z"/>
    <path d="M0-.5h24v24H0z" fill="none"/>
  </svg>,
  down: <svg fill="#000000" height="22" width="22" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">
    <path d="M7.41 7.84L12 12.42l4.59-4.58L18 9.25l-6 6-6-6z"/>
    <path d="M0-.75h24v24H0z" fill="none"/>
  </svg>,
  up: <svg fill="#000000" height="22" width="22" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">
    <path d="M7.41 15.41L12 10.83l4.59 4.58L18 14l-6-6-6 6z"/>
    <path d="M0 0h24v24H0z" fill="none"/>
  </svg>
}

export default class Splitter extends Component {

  prepareChildren(props) {
    const result = []
    const horizontal = props.orientation == 'horizontal'

    const className = `${CLASS_NAME}-inner`
    const row = horizontal
    const column = !row

    const flexProps = {
      justifyContent: 'center',
      alignItems: 'center',
      wrap: false,
      row,
      column
    }

    const { collapsedIndex, locked } = props

    if (locked){
      return null
    }

    const firstCollapsed = collapsedIndex == 0
    const secondCollapsed = collapsedIndex == 1

    const collapsible = props.collapsible || []
    const firstCollapsible = collapsible[0] !== false
    const secondCollapsible = collapsible[1] !== false

    if (firstCollapsible && !firstCollapsed || secondCollapsed){
        result.push(
          <Flex
            {...flexProps}
            key="first"
            className={join(className, `${className}--first`, `${className}--orientation-${props.orientation}`)}
            onClick={this.toggleCollapse.bind(this, -1)}
          >
            {horizontal? ARROWS.up: ARROWS.left}
          </Flex>
        )
    }

    if (secondCollapsible && !secondCollapsed || firstCollapsed){
      result.push(
        <Flex
          {...flexProps}
          key="second"
          className={join(className, `${className}--second`, `${className}--orientation-${props.orientation}`)}
          onClick={this.toggleCollapse.bind(this, 1)}
        >
          {horizontal? ARROWS.down: ARROWS.right}
        </Flex>
      )
    }

    return result
  }

  prepareProps(thisProps){
    const props = assign({}, this.props)

    props.className = this.prepareClassName(props)
    props.children = this.prepareChildren(props)

    return props
  }

  prepareClassName(props){

    const { collapsedIndex, collapsing, expanding, locked, draggable, constrained } = props

    return join(
      props.className,
      CLASS_NAME,
      `${CLASS_NAME}--orientation-${props.orientation}`,

      locked && `${CLASS_NAME}--locked`,
      draggable && `${CLASS_NAME}--draggable`,
      constrained && `${CLASS_NAME}--constrained`,

      collapsedIndex != undefined &&
        join(
          `${CLASS_NAME}--collapsed`,
          `${CLASS_NAME}--collapsed-${collapsedIndex == 0? 'start': 'end'}`
        )
    )
  }

  render() {

      const props = this.prepareProps(this.props)

      const horizontal = props.orientation == 'horizontal'

      const row = horizontal
      const column = !row

      return <Flex
        wrap={false}
        row={row}
        column={column}
        justifyContent="center"
        alignItems="center"
        {...props}
        draggable={null}
        onDoubleClick={this.handleDoubleClick}
        onMouseDown={this.handleMouseDown}
      />
  }

  handleDoubleClick(event){
    this.props.onDoubleClick(event)

    const { collapsedIndex } = this.props

    if (collapsedIndex == null){
      return
    }

    this.toggleCollapse(collapsedIndex == 0? 1: -1)
  }

  handleMouseDown(event) {

    event.preventDefault()

    this.props.onMouseDown(event)

    if (this.props.locked || !this.props.draggable || this.props.collapsedIndex != null){
      return
    }

    const node = findDOMNode(this)
    const parent = node.parentNode
    const zeroPositionName = this.props.orientation == 'vertical'? 'top': 'left'

    DragHelper(event, {
      scope: this,
      region: node,
      constrainTo: parent,
      onDragStart: function(event, config) {
        config.diff[zeroPositionName] = 0
        this.props.onDragStart(config.diff, event)
      },

      onDrag: function(event, config) {
        config.diff[zeroPositionName] = 0
        this.props.onDrag(config.diff, event)
      },

      onDrop: function(event, config) {
        config.diff[zeroPositionName] = 0
        this.props.onDrop(config.diff, event)
      }
    })
  }

  toggleCollapse(direction, event){

      // event.stopPropagation()

      if (this.props.locked){
          return
      }

      const {
        collapsedIndex,
        onExpand,
        onCollapse
      } = this.props

      const index = collapsedIndex != undefined?
                      //there is a collapsed item
                      //so we are expanding
                      collapsedIndex:

                      //otherwise, we are collapsing
                      direction == 1?
                        1: 0

      const fn = collapsedIndex === index?
                  onExpand:
                  onCollapse

      fn(index, event)
  }
}

Splitter.propTypes = {
  size: PropTypes.number,
  orientation: PropTypes.string,
  locked: PropTypes.bool,

  collapsedIndex: PropTypes.oneOf([undefined, 0, 1])
}

Splitter.defaultProps = {
  isSplitter: true,

  onDragStart: emptyFn,
  onDrag: emptyFn,
  onDrop: emptyFn,

  onCollapse: emptyFn,
  onExpand: emptyFn,

  onDoubleClick: emptyFn,
  onMouseDown: emptyFn,
  beforeMin: null,
  afterMin: null
}
