import React, { PropTypes } from 'react'
import { findDOMNode } from 'react-dom'

import Component from 'react-class'

import raf from 'raf'

import { Flex, Item } from 'react-flex'

import assign from 'object-assign'
import normalize from 'react-style-normalizer'

import Splitter from './Splitter'
import join from './join'
import bemFactory from './bemFactory'
import isNumber from './isNumber'
import calc from './calc'

import getTransitionEnd from './getTransitionEnd'

const PERCENTAGE_RE = /%/
const CLASS_NAME = 'react-split-container'

const minMax = (min, max) => {
  return {
    min: Math.min(min, max),
    max: Math.max(min, max)
  }
}

const clamp = (n, min, max) => {
  return Math.max(Math.min(n, max), min)
}

const bem = bemFactory(CLASS_NAME)
const m = (name) => bem(null, name)

const prefixFlexGrow = (value) => {
  return {
    WebkitBoxFlex: value,
    WebkitFlexGrow: value,
    MozFlexGrow: value,
    MsFlexPositive: value,
    flexGrow: value
  }
}

const prefixFlex = (value) => {
  return {
    WebkitBoxFlex: value,
    WebkitFlex: value,
    MozFlex: value,
    MsFlex: value,
    flex: value
  }
}

const isPercentage = (n) => {
  if (isNumber(n)){
      return false
  }

  return typeof n == 'string' && isNumber(n.replace(PERCENTAGE_RE, ''))
}

const getSizeName = (props) => {
  return props.orientation == 'horizontal'?
          'height': 'width'
}

const getOtherSizeName = (props) => {
  return props.orientation == 'horizontal'?
          'width': 'height'
}

const getMaxSizeName = (props) => {
  return props.orientation == 'horizontal'?
          'maxHeight': 'maxWidth'
}

const getMinSizeName = (props) => {
  return props.orientation == 'horizontal'?
          'minHeight': 'minWidth'
}

const getPositionNames = (props) => {
  var H = props.orientation == 'horizontal'
  return H? ['top', 'bottom']: ['left','right']
}

const toUpperFirst = (str) => {
  return typeof str === 'string'?
    str[0].toUpperCase() + str.substring(1):
    ''
}

const getOffsetSizeName = (props) => {
  const sizeName = getSizeName(props)

  return 'client' + toUpperFirst(sizeName)
}

const emptyFn = () => {}

const getOffsetSize = node => {

  if (node.getBoundingClientRect){
    const rect = node.getBoundingClientRect()

    return {
      width: rect.width,
      height: rect.height
    }
  }

  return {
    width: node.offsetWidth,
    height: node.offsetHeight
  }
}

const inDevMode = /param/.test(function(param) {});
/**
 * node is the flex wrapper, node.firstChild is the Side, and node.firstChild.firstChild is the actual node
 * we are measuring the auto size for
 */
const getAutoSize = (node) => {
  const targetNode = node.firstChild.firstChild

  const size = getOffsetSize(targetNode)
  const borderPadding = getBorderPaddingSize(node.firstChild)

  return {
    width: size.width + borderPadding.left + borderPadding.right,
    height: size.height + borderPadding.top + borderPadding.bottom
  }
}

const toNumber = (n) => parseInt(n, 10)

const getBorderPaddingSize = (node) => {
  const computedStyle = global.getComputedStyle(node)

  return {
    left: toNumber(computedStyle.borderLeftWidth) + toNumber(computedStyle.paddingLeft),
    right: toNumber(computedStyle.borderRightWidth) + toNumber(computedStyle.paddingRight),
    top: toNumber(computedStyle.borderTopWidth) + toNumber(computedStyle.paddingTop),
    bottom: toNumber(computedStyle.borderBottomWidth) + toNumber(computedStyle.paddingBottom)
  }
}

class SplitSide extends Component {
  render(){
    return <Item {...this.props} />
  }
}

SplitSide.propTypes = {
  children: (props, propsName) => {
    if (props.index == props.splitAtIndex && props.splitAtValue == 'auto' && React.Children.toArray(props.children).length > 1){
      return new Error(`The SplitContainer side at index ${props.splitAtIndex} has "auto" splitValue but has multiple children. It should only have 1 child!`)
    }
  }
}

class Side extends Component {
  render(){
    return null
  }
}

Side.defaultProps = {
  isSplitContainerSide: true
}

export { Splitter, Side }

const TRANSITION_DURATION = '0.4s'

export default class SplitContainer extends Component {

  constructor(props){
    super(props)

    let collapsedIndex = props.defaultCollapsedIndex

    if (collapsedIndex != 0 && collapsedIndex != 1){
      collapsedIndex = null
    }

    this.state = {
      dragging: false,
      splitAt: props.defaultSplitAt,
      collapsedIndex
    }
  }

  onTransitionEnd(event){
    event && event.target && event.target.removeEventListener(getTransitionEnd(), this.onTransitionEnd, false)

    this.setState({
      collapsing: false,
      expanding: false,

      collapsingIndex: null,
      expandingIndex: null,
      constrainingIndex: null,

      styles: null,

      expandStyle: null,
      collapseStyle: null,

      splitterStyle: null
    })
  }

  hasTransition(props){
    return !!(props.transition || props.transitionDuration)
  }

  componentWillReceiveProps(nextProps){

    const props = this.p

    const collapsedIndex = props.collapsedIndex
    const nextCollapsedIndex = this.prepareCollapsedIndex(nextProps)

    this.doTransition(nextProps, collapsedIndex, nextCollapsedIndex)
  }

  doTransition(props, oldCollapsed, newCollapsed){
    const collapsing = oldCollapsed == null && (newCollapsed == 0 || newCollapsed == 1)
    const expanding = (oldCollapsed == 0 || oldCollapsed == 1) && newCollapsed == null

    if (collapsing){
      this.transitionCollapse(props, newCollapsed)
    } else if (expanding){
      this.transitionExpand(props, oldCollapsed)
    }
  }

  getNodes(nodes){
    if (nodes){
      return nodes.map(findDOMNode)
    }

    return [
      findDOMNode(this.first),
      findDOMNode(this.second),
      findDOMNode(this.splitter)
    ]
  }

  transitionExpand(props, expandIndex){

    const nodes = this.getNodes()

    const transitionDuration = normalize({transitionDuration: props.transitionDuration || TRANSITION_DURATION})

    const constrainingIndex = 1 - expandIndex

    const sizeName = getSizeName(props)
    const otherSizeName = getOtherSizeName(props)

    const sideName = props.orientation == 'horizontal'?
                      constrainingIndex == 1? 'top': 'bottom':
                      constrainingIndex == 1? 'left': 'right'

    const otherSideName = props.orientation == 'horizontal'?
                      constrainingIndex == 1? 'left': 'right':
                      constrainingIndex == 1? 'top': 'bottom'

    const marginName = 'margin' + toUpperFirst(sideName)

    const { splitAtIndex, asPercentage } = props
    let { splitAtValue } = props

    const sizes = nodes.map(getOffsetSize)

    if (splitAtValue == 'auto'){
      sizes[splitAtIndex] = getAutoSize(nodes[splitAtIndex])
    }

    const styles = []

    const splitterSize = sizes[2][sizeName]

    const totalSize = getOffsetSize(nodes[0])[sizeName] + getOffsetSize(nodes[1])[sizeName]

    if (splitAtValue == 'auto'){
      splitAtValue = sizes[splitAtIndex][sizeName]
    }

    let expandSize = splitAtValue

    if (asPercentage){
      expandSize = splitAtValue * (totalSize) / 100
    }

    if (expandIndex == 1){
      expandSize = totalSize - expandSize
    }

    if (splitAtIndex == 1){
      expandSize = totalSize - expandSize
    }

    styles[constrainingIndex] = {
      [marginName]: sizes[2][sizeName],
      //keep the size so in case everything is autosized it wont be collapsed to a smaller size
      [otherSizeName]: sizes[2][otherSizeName]
    }

    styles[expandIndex] = {
      position: 'absolute',
      zIndex: 1000,
      [otherSizeName]: sizes[2][otherSizeName],
      [sizeName]: expandSize,
      [sideName]: -expandSize,
      [otherSideName]: 0
    }

    const splitterStyle = {
      position: 'absolute',
      zIndex: 1000,
      [sideName]: 0,
      [otherSideName]: 0
    }

    this.setState({
      expandingIndex: expandIndex,
      styles,
      splitterStyle
    }, () => {

      raf(() => {
        if (!expandSize){
          if (props.splitAtValue == 'auto'){
            inDevMode && console && console.warn && console.warn(`You are using 'splitAt="auto"' but your content in side ${constrainingIndex} is probably 'display:"block"'.
              Use 'display: "inline-block"' for transitions on expand/collapse to work properly.`)
          }

          return this.onTransitionEnd()
        }

        const expandNode = nodes[expandIndex]

        expandNode.addEventListener(getTransitionEnd(), this.onTransitionEnd, false)

        raf(() => {

          styles[expandIndex] = assign({}, styles[expandIndex], {
            [sizeName]: expandSize,
            [sideName]: 0
          }, transitionDuration)

          this.setState({

            splitterStyle: assign({}, splitterStyle, {
              [sideName]: expandSize
            }, transitionDuration),

            styles
          })

        })
      })
    })
  }

  transitionCollapse(props, nextCollapsedIndex){

    const nodes = this.getNodes()

    const expandIndex = 1 - nextCollapsedIndex
    const collapsingIndex = nextCollapsedIndex

    const sizeName = getSizeName(props)
    const otherSizeName = getOtherSizeName(props)

    const sideName = props.orientation == 'horizontal'?
                      nextCollapsedIndex == 1? 'bottom': 'top':
                      nextCollapsedIndex == 1? 'right': 'left'
    const otherSideName = props.orientation == 'horizontal'?
                      nextCollapsedIndex == 1? 'right': 'left':
                      nextCollapsedIndex == 1? 'bottom': 'top'


    const marginName = 'margin' + toUpperFirst(sideName)

    const transitionDuration = normalize({
      transitionDuration: props.transitionDuration || TRANSITION_DURATION,
      transitionProperty: sideName
    })

    const sizes = nodes.map(getOffsetSize)

    const styles = []

    styles[expandIndex] = {
      [marginName]: sizes[2][sizeName],
      //keep the size so in case everything is autosized, this wont collapse to a smaller size
      [otherSizeName]: sizes[expandIndex][otherSizeName]
    }

    styles[collapsingIndex] = {
      position: 'absolute',
      zIndex: 1000,
      [otherSizeName]: sizes[collapsingIndex][otherSizeName],

      [sizeName]: sizes[collapsingIndex][sizeName],

      [sideName]: 0,
      [otherSideName]: 0
    }

    const splitterStyle = {
      position: 'absolute',
      zIndex: 1000,

      width: sizes[2].width,
      height: sizes[2].height,

      [sideName]: sizes[collapsingIndex][sizeName],
      [otherSideName]: 0
    }

    this.setState({
      collapsing: true,

      collapsingIndex,

      sizes,
      styles,

      splitterStyle

    }, () => {

      raf( () => {
        setTimeout(() => {
          // debugger
        })
        const collapseNode = nodes[collapsingIndex]

        collapseNode.addEventListener(getTransitionEnd(), this.onTransitionEnd, false)

        raf(() => {
          styles[collapsingIndex] = assign({}, styles[collapsingIndex], {
            [sideName]: -sizes[collapsingIndex][sizeName]
          }, transitionDuration)

          this.setState({
            splitterStyle: assign({}, splitterStyle, {
              [sideName]: 0
            }, transitionDuration),
            styles
          })
        })
      })
    })
  }

  prepareChildren(props){
    let splitterProps
    let firstProps
    let secondProps

    const children = []

    React.Children
      .toArray(props.children)
      .forEach((child, index) => {
        if (child && child.props){

          if (child.props.isSplitter){
            splitterProps = child.props
            return
          }

          if (child.props.isSplitContainerSide){
            if (index == 1){
              secondProps = child.props
            }
            if (index == 0){
              firstProps = child.props
            }

            children.push(child.props.children)

            return
          }
        }

        children.push(child)
      })

    return {
      children,
      firstProps,
      secondProps,
      splitterProps
    }
  }

  prepareCollapsedIndex(props, state = this.state){
    let collapsedIndex = props.collapsedIndex === undefined?
                            state.collapsedIndex:
                            props.collapsedIndex

    if (collapsedIndex !== 0 && collapsedIndex !== 1){
      collapsedIndex = undefined
    }

    return collapsedIndex
  }

  prepareClassNames(props){
    const { children, collapsedIndex } = props

    const firstClassName = children[0] && children[0].props && children[0].props.className
    const secondClassName = children[1] && children[1].props && children[1].props.className

    const expandingIndex = this.state.expandingIndex
    const constrainingIndex = this.state.constrainingIndex

    return [
      join(
        firstClassName,

        `${CLASS_NAME}__side`,
        `${CLASS_NAME}__side--first`,

        collapsedIndex == 0?
          this.state.collapsing?
            `${CLASS_NAME}__side--collapsing`:
            `${CLASS_NAME}__side--collapsed`
          :
          '',

        expandingIndex == 0 && `${CLASS_NAME}__side--expanding`,
        constrainingIndex == 0 && `${CLASS_NAME}__side--constraining`,

        `${CLASS_NAME}__side--orientation-${props.orientation}`
      ),

      join(
        secondClassName,

        `${CLASS_NAME}__side`,
        `${CLASS_NAME}__side--second`,

        collapsedIndex == 1?
          this.state.collapsing?
            `${CLASS_NAME}__side--collapsing`:
            `${CLASS_NAME}__side--collapsed`
          :
          '',

        expandingIndex == 1 && `${CLASS_NAME}__side--expanding`,
        constrainingIndex == 1 && `${CLASS_NAME}__side--constraining`,

        `${CLASS_NAME}__side--orientation-${props.orientation}`
      )
    ]
  }

  prepareSplitAt(props){
    const result = props.splitAt == null?
            this.state.splitAt:
            props.splitAt

    if (result == null){
      return 'auto'
    }

    return result
  }

  prepareProps(thisProps){
    const props = this.p = assign({}, thisProps)

    const { children, splitterProps, firstProps, secondProps } = this.prepareChildren(props)

    props.collapsedIndex = this.prepareCollapsedIndex(props)

    if (props.collapsedIndex !== 0 && props.collapsedIndex !== 1){
      delete props.collapsedIndex
    } else {
      props.notCollapsedIndex = 1 - props.collapsedIndex
    }

    if (firstProps){
      props.firstProps = firstProps
    }

    if (secondProps){
      props.secondProps = secondProps
    }

    props.children = children
    props.splitterProps = splitterProps

    props.horizontal = props.orientation == 'horizontal'
    props.vertical = !props.orientation

    props.splitAt = this.prepareSplitAt(props)
    props.className = this.prepareClassName(props)

    props.sizeName = getSizeName(props)
    props.otherSizeName = getOtherSizeName(props)
    props.minSizeName = getMinSizeName(props)
    props.maxSizeName = getMaxSizeName(props)

    props.asPercentage = isPercentage(props.splitAt)

    const { splitAt } = props

    let splitAtIndex = 0
    let splitAtValue

    if (splitAt == 'auto'){
      splitAtValue = 'auto'
    } else if (splitAt == '-auto'){
      splitAtValue = 'auto'
      splitAtIndex = 1
    } else {
      splitAtValue = parseFloat(splitAt, 10)

      if (splitAtValue < 0){
        if (props.asPercentage){
          splitAtValue = 100 - (100 + splitAtValue)
        } else {
          splitAtValue *= -1
        }

        splitAtIndex = 1
      }
    }

    props.splitAtIndex = splitAtIndex
    props.splitAtValue = splitAtValue

    let { fixedIndex } = props

    if (fixedIndex != 0 && fixedIndex != 1){
      fixedIndex = -1
    }

    let flexIndex = 1 - fixedIndex

    props.fixedIndex = fixedIndex
    props.flexIndex = flexIndex

    props.styles = this.prepareStyles(props)
    props.classNames = this.prepareClassNames(props)
    props.splitterStyle = this.prepareSplitterStyle(props)

    return props
  }

  prepareSplitterStyle(props){
    const style = this.state.splitterStyle

    return style
  }

  prepareStyles(props){
    const state = this.state

    const styles = [
      {},
      {}
      //TODO - use those if you don't have all flex classes from 1 to 100
      // prefixFlexGrow('none'),
      // prefixFlexGrow('none')
    ]

    const sizeProps = props.sizeProps = [
      {
        flex: 'none'
      },
      {
        flex: 'none'
      }
    ]

    const {
      collapsedIndex,
      notCollapsedIndex,
      splitAtIndex,
      splitAtValue,
      flexIndex,
      fixedIndex
    } = props

    const otherIndex = 1 - splitAtIndex

    const splitAtStyle = styles[splitAtIndex]
    const otherStyle = styles[otherIndex]

    const hasCollapsed = collapsedIndex != undefined

    if (props.asPercentage){

      // const percentage = splitAtIndex == 1?
      // if (splitAtIndex )
      sizeProps[splitAtIndex].flex = splitAtValue
      sizeProps[otherIndex].flex = 100 - splitAtValue
      //TODO - use those instead of the above when you don't have all classes from 1 to 100
      // assign(styles[splitAtIndex], prefixFlexGrow(splitAtValue))
      // assign(styles[otherIndex], prefixFlexGrow(100 - splitAtValue))
    } else {

      sizeProps[splitAtIndex].flex = 'none'
      sizeProps[otherIndex].flex = 1

      styles[splitAtIndex][props.sizeName] = splitAtValue
    }

    if (hasCollapsed){
      styles[collapsedIndex] = {
        [props.sizeName]: 0
      }

      sizeProps[collapsedIndex].flex = 'none'
      sizeProps[notCollapsedIndex].flex = 1
    }

    if (this.state.expandingIndex != null){
      sizeProps[1 - this.state.expandingIndex].flex = 1
    }

    const stateStyles = this.state.styles

    if (stateStyles){
      if (stateStyles[0]){
        styles[0] = stateStyles[0]
      }

      if (stateStyles[1]){
        styles[1] = stateStyles[1]
      }
    }

    assign(styles[0], { overflow: 'hidden' })
    assign(styles[1], { overflow: 'hidden' })

    sizeProps[0].style = styles[0]
    sizeProps[1].style = styles[1]

    return styles
  }

  prepareClassName(props){
    return join(
      CLASS_NAME,
      m(props.orientation),
      m(`orientation-${props.orientation}`),
      props.theme && m(`theme-${props.theme}`),
      props.collapsing && m(`collapsing`),
      props.locked && m('locked'),
      this.state.dragging && m('resizing'),
      this.state.constrained && m('constrained')
    )
  }

  render() {
    const props = this.p = this.prepareProps(this.props)

    const row = props.vertical
    const column = props.horizontal

    const {
      firstProps,
      secondProps,

      splitterProps,

      collapsedIndex,
      notCollapsedIndex
    } = props

    const sizeProps = props.sizeProps

    const newProps = [
      {
        flexShrink: 0,
        flexBasis: 0,
        className: props.classNames[0],
        children: this.renderFirst()
      },
      {
        flexShrink: 0,
        flexBasis: 0,
        className: props.classNames[1],
        children: this.renderSecond()
      }
    ]

    if (props.splitAtValue == 'auto' && props.splitAtIndex === collapsedIndex && this.state.collapsingIndex == null){
      //when we have auto size for a region, in case it is collapsed
      //we need the Item to have flex none to allow the content be sized as is
      //and not wrap to another size
      newProps[props.splitAtIndex].flex = 'none'
    }

    const splitterStyle = assign({}, (splitterProps || {}).style, props.splitterStyle)

    return <Flex
      {...this.props}
      draggable={null}
      row={row}
      column={column}
      wrap={false}
      alignItems="stretch"
      className={props.className}
    >
      <Flex {...sizeProps[0]} alignSelf="stretch" alignItems="stretch" ref={c => this.first = c}>
        <SplitSide
          {...props.firstProps}
          {...newProps[0]}
          index={0}
          splitAtValue={props.splitAtValue}
          splitAtIndex={props.splitAtIndex}
        />
      </Flex>

      <Splitter
        {...splitterProps}

        style={splitterStyle}
        ref={s => this.splitter = s}

        onDragStart={this.handleSplitterDragStart}
        onDrag={this.handleSplitterDrag}
        onDrop={this.handleSplitterDrop}

        onCollapse={this.handleCollapse}
        onExpand={this.handleExpand}

        orientation={props.orientation}

        locked={props.locked}
        draggable={props.draggable}

        constrained={!props.resizeProxy && this.state.constrained}

        collapsedIndex={props.collapsedIndex}

        collapsible={props.collapsible}
      />

      <Flex {...sizeProps[1]} alignSelf="stretch" alignItems="stretch" ref={c => this.second = c}>
        <SplitSide
          {...props.secondProps}
          {...newProps[1]}
          index={1}
          splitAtValue={props.splitAtValue}
          splitAtIndex={props.splitAtIndex}
        />
      </Flex>

      {this.props.resizeProxy && this.renderResizeProxy()}
      {this.renderCover()}
    </Flex>
  }

  renderCover(){

    if (this.props.renderCover){
      return this.props.renderCover()
    }

    if (!this.props.cover){
      return null
    }

    const className = join(
      `${CLASS_NAME}__cover`,
      this.state.dragging && `${CLASS_NAME}__cover--resizing`
    )

    return <div
      className={className}
      style={{
        display: this.state.dragging? 'block': 'none',
        position: 'absolute',
        zIndex: 1000,
        width: '100%',
        height: '100%',
        left: 0,
        top: 0
      }}
    />
  }

  renderResizeProxy(){
    const props = this.p
    const positionNames = getPositionNames(props)

    const style = {
      display: this.state.dragging? 'block': 'none',
      [props.otherSizeName]: '100%',
      [props.sizeName]: this.splitterSize,
      position: 'absolute',
      zIndex: 1000,
      [positionNames[0]]: this.state.dragPosition,
      [props.orientation == 'horizontal'? 'left': 'top']: 0
    }

    const className = join(
      `${CLASS_NAME}__resize-proxy`,
      this.state.constrained && `${CLASS_NAME}__resize-proxy--constrained`
    )

    return <div className={className} style={style}/>
  }

  renderFirst(){
    return this.renderAt(0)
  }

  renderSecond(){
    return this.renderAt(1)
  }

  renderAt(index){
    const children = this.p.children

    return children[index]
  }

  constrain(value, { min, max }){

    const { constrain } = this.props

    if (constrain === false){
      return {
        value,
        consrained: false
      }
    }

    if (typeof constrain == 'function'){
      return this.props.constrain(value, { min, max })
    }

    const result = clamp(value, min, max)

    return {
      value: result,
      constrained: result !== value
    }
  }

  handleSplitterDragStart(diff, event){

    const props = this.p
    const fn = props[props.onDragStartName]

    if (typeof fn == 'function'){
      fn(diff, event)
    }

    const nodes = this.getNodes()
    const sizes = nodes.map(getOffsetSize)

    this.sizes = sizes
    this.splitterPosition = sizes[0][props.sizeName]
    this.splitterSize = sizes[2][props.sizeName]

    const minSize = props.minSize || []
    const maxSize = props.maxSize || []

    const available = sizes[0][props.sizeName] + sizes[1][props.sizeName]

    let splitterMin = 0
    let splitterMax = available

    if (minSize[0]){
      splitterMin += minSize[0]
    }
    if (minSize[1]){
      splitterMax -= minSize[1]
    }

    if (maxSize[0]){
      splitterMax = Math.min(splitterMax, maxSize[0])
    }

    if (maxSize[1]){
      splitterMin = Math.max(splitterMin, available - maxSize[1])
    }

    const values = minMax(splitterMin, splitterMax)

    this.splitterMin = values.min
    this.splitterMax = values.max

  }

  handleSplitterDrag(diff, event) {

    const props = this.p

    const positions = getPositionNames(this.props)
    const posName = positions[0]
    const offset = diff[posName]

    if (!offset){
      return
    }

    const fn = this.props[this.props.onDragName]

    if (typeof fn == 'function'){
      fn(diff, event)
    }


    let dragPosition = this.splitterPosition + offset

    const { value, constrained } = this.constrain(dragPosition, {
      min: this.splitterMin,
      max: this.splitterMax
    })

    if (!props.resizeProxy){

      this.doResize(value, { constrain: false })

      if (this.state.constrained != constrained){
        this.setState({ constrained })
      }

    } else {

      this.setState({
        dragging: true,
        dragPosition: value,
        constrained
      })

    }

  }

  handleSplitterDrop(diff, event){

    const props = this.p
    const positions = getPositionNames(props)
    const posName = positions[0]

    const offset = diff[posName]
    const position = this.splitterPosition + offset

    if (!offset){

      delete this.sizes
      delete this.splitterPosition
      delete this.splitterSize

      return
    }

    const state = {
      dragging: false,
      dragPosition: null,
      constrained: false
    }

    this.setState(state)

    this.doResize(position)

    delete this.sizes
    delete this.splitterPosition
    delete this.splitterSize
  }

  doResize(at, { constrain = true } = { constrain: true}){
    const props = this.p

    let constrained = false
    let value = at

    if (constrain){
      const result = this.constrain(at, {
        min: this.splitterMin,
        max: this.splitterMax
      })

      value = result.value
      constrained = result.constrained
    }

    const { splitAtIndex, sizeName } = props

    const sizes = this.sizes
    const sign = splitAtIndex == 1? -1: 1

    if (splitAtIndex == 1){
      value = sizes[0][sizeName] + sizes[1][sizeName] - value
    }

    value *= sign

    if (this.props.splitAt == null){
      this.setState({
        splitAt: value
      })
    }

    this.props.onResize(value)
  }

  handleCollapse(collapsedIndex){
    if (this.props.collapsedIndex === undefined){
      this.setState({
        collapsedIndex
      })

      this.doTransition(this.p, this.p.collapsedIndex, collapsedIndex)
    }

    this.props.onCollapse(collapsedIndex)
  }

  handleExpand(expandIndex){

    const collapsedIndex = this.p.collapsedIndex

    if (expandIndex !== collapsedIndex){
      //invalid expand
      return
    }

    if (this.props.collapsedIndex === undefined){
      this.setState({
        collapsedIndex: null
      })

      this.doTransition(this.p, collapsedIndex, null)
    }

    this.props.onExpand(expandIndex)
  }

}

SplitContainer.propTypes = {
  children: function(props, propName){
    var propValue = React.Children.toArray(props[propName])

    propValue = propValue.filter(child => {
      if (child && child.props && child.props.isSplitter){
        return null
      }
      return child
    })

    const isArray = Array.isArray(propValue)
    const isTwo = isArray && propValue.length == 2

    if (!propValue || !isArray || !isTwo){
      console.warn('Please pass only 2 children. You provided ' + (isArray? propValue.length + ' children': propValue))
    }
  },

  orientation: PropTypes.oneOf(['horizontal','vertical']),

  locked: PropTypes.bool,
  draggable: PropTypes.bool,

  splitAt: (props, propName) => {
    const value = props[propName]
    const fixedIndex = props.fixedIndex

    if (isPercentage(value) && (fixedIndex == 0 || fixedIndex == 1)){
      return new Error('"fixedIndex" should be used when "spliAt" is a expressed as a fixed number, and not as a percentage')
    }
  }
}

SplitContainer.defaultProps = {
  theme: 'default',

  fixedIndex: -1,
  orientation: 'horizontal',

  cover: true,
  draggable: true,
  locked: false,

  resizeProxy: true,

  onCollapse: () => {},
  onExpand: () => {},
  onResize: () => {},

  collapsible: [true, true]
}
